/*
 * Provides an average of values.
 *
 * Author: J. Lewis Muir
 */

#ifndef AMY_AVERAGE_H
#define AMY_AVERAGE_H

#define  P_ValueIntString      "VALUE_INT"      /* asynInt32,   rw */
#define  P_ValueFltString      "VALUE_FLT"      /* asynFloat64, rw */
#define  P_ArithMeanIntString  "ARITH_MEAN_INT" /* asynInt32,   r  */
#define  P_ArithMeanFltString  "ARITH_MEAN_FLT" /* asynFloat64, r  */
#define  P_MovingString        "MOVING"         /* asynInt32,   rw */
                                                /* 0=no, 1=yes     */
#define  P_ResetString         "RESET"          /* asynInt32,    w */
                                                /* value ignored   */
#define  P_WindowSizeString    "WINDOW_SIZE"    /* asynInt32,   rw */
#define  P_MaxNumValuesString  "MAX_NUM_VALUES" /* asynInt32,   r  */

class AmyAverage : public asynPortDriver {
public:
  AmyAverage(const char *portName, int numAddrs, int maxNumValues,
    int integerStorageAndArithmetic=0);
  virtual ~AmyAverage();

  virtual asynStatus writeInt32(asynUser *pasynUser, epicsInt32 value);
  virtual asynStatus writeFloat64(asynUser *pasynUser, epicsFloat64 value);

protected:
  int P_ValueInt;
  #define FIRST_AMY_AVERAGE_COMMAND P_ValueInt
  int P_ValueFlt;
  int P_ArithMeanInt;
  int P_ArithMeanFlt;
  int P_Moving;
  int P_Reset;
  int P_WindowSize;
  int P_MaxNumValues;
  #define LAST_AMY_AVERAGE_COMMAND P_MaxNumValues

private:
  int integerStorageAndArithmetic;
  int *windowIndex;
  int *windowFull;
  epicsInt32 **integerValues;
  epicsFloat64 **floatValues;

  void setValue(int addr, epicsInt32 integerValue, epicsFloat64 floatValue);
  void computeAndSetArithMeanIfNeeded(int addr);
  epicsInt32 computeIntegerArithMean(int addr, int regionSize);
  epicsFloat64 computeFloatArithMean(int addr, int regionSize);
  int isMoving(int addr);
  void setMoving(int addr, int moving);
  void reset(int addr);
  epicsInt32 getWindowSize(int addr);
  void setWindowSize(int addr, epicsInt32 size);
  epicsInt32 getMaxNumValues(int addr);
};

#define NUM_AMY_AVERAGE_PARAMS (&LAST_AMY_AVERAGE_COMMAND - \
  &FIRST_AMY_AVERAGE_COMMAND + 1)

#ifdef __cplusplus
extern "C" {
#endif

int amyAverageConfig(const char *portName, int numAddrs, int maxNumValues,
  int integerStorageAndArithmetic);

#ifdef __cplusplus
}
#endif

#endif
